/*
    TCFbERotatedTableEx.cpp    December 4, 2004.

    Copyright (C) 2003-2005 CFbE Research Group,
    Software Engineering Laboratory,
    Graduate School of Information Science,
    Nara Institute of Science and Technology,
    All rights reserved.

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, (at your option) or
    any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GNU Emacs; see the file COPYING.  If not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
//---------------------------------------------------------------------------
#pragma hdrstop

#include "TCFbERotatedTableEx.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// コンストラクタ（空キャッシュを作成
__fastcall TCFbERotatedTableEx::TCFbERotatedTableEx(void)
    : TCFbECustomTableEx()
{
}

//---------------------------------------------------------------------------
// 指定された行列を備えた空テーブルを作成
__fastcall TCFbERotatedTableEx::TCFbERotatedTableEx(TStringList* RowLabelList, TStringList* ColumnLabelList)
    : TCFbECustomTableEx(RowLabelList, ColumnLabelList)
{
}

//---------------------------------------------------------------------------
// コンストラクタ
__fastcall TCFbERotatedTableEx::TCFbERotatedTableEx(TStringList* SourceCsvStringList)
    : TCFbECustomTableEx(SourceCsvStringList)
{
}

//---------------------------------------------------------------------------
// デストラクタ
__fastcall TCFbERotatedTableEx::~TCFbERotatedTableEx()
{
}

//---------------------------------------------------------------------------
// 親キャッシュのセルが更新された
void __fastcall TCFbERotatedTableEx::ParentCellIsUpdated(int RowIndex, int ColumnIndex)
{
    TCFbECustomTableEx::ParentCellIsUpdated(ColumnIndex, RowIndex);
}

//---------------------------------------------------------------------------
// 親キャッシュの行が追加された
int __fastcall  TCFbERotatedTableEx::ParentAddRow(const char* RowLabel)
{
    return this->AddColumn(RowLabel);
}

//---------------------------------------------------------------------------
// 親キャッシュの列が追加された
int __fastcall  TCFbERotatedTableEx::ParentAddColumn(const char* ColumnLabel)
{
    return this->AddRow(ColumnLabel);
}

//---------------------------------------------------------------------------
// 親キャッシュの行が削除された
void __fastcall TCFbERotatedTableEx::ParentDeleteRowByIndex(int TargetRowIndex)
{
    this->DeleteColumnByIndex(TargetRowIndex);
}

//---------------------------------------------------------------------------
// 親キャッシュの列が削除された
void __fastcall TCFbERotatedTableEx::ParentDeleteColumnByIndex(int TargetColumnIndex)
{
    this->DeleteRowByIndex(TargetColumnIndex);
}

//---------------------------------------------------------------------------
// 以下，protected method
//---------------------------------------------------------------------------
// 自分自身のクローンを作成する
TCFbECachedTable* __fastcall TCFbERotatedTableEx::CreateClone(void)
{
    TCFbERotatedTableEx* Clone = new TCFbERotatedTableEx();
    Clone->Assign(this);
    return Clone;
}

//---------------------------------------------------------------------------
// 自分自身の行と列が入れ替わったクローンを作成する
TCFbECachedTable* __fastcall TCFbERotatedTableEx::CreateRotatedClone(void)
{
    TCFbETableEx* Clone = new TCFbETableEx();
    Clone->RotatedAssign(this);
    return Clone;
}

//---------------------------------------------------------------------------

